

#include "Arduino.h"
#include "nrf.h"
#include "nrf52_pwm.h"
#include <math.h>  

#define NSAMPLES 256 // nombre d'intervalles mémorisés
#define NBITS 16// nombre de bits dans le séquence PWM
#define OUTPUT_PIN 3
#define INPUT_PIN 4
#define DECODE_BITS

uint8_t bits[16] = {0,0,1,0,1,1,0,1,0,0,0,1,0,1,1,1};
uint16_t sequence_0[NBITS];
uint16_t sequence_1[NBITS];
uint32_t t1,t2;
int32_t intervalles[NSAMPLES];
uint8_t lecture_bits[NSAMPLES]; // pour le décodage des bits
int16_t indice_t; // indice pour le remplissage du tableau intervalles
uint8_t prescaler = 0; // horloge du PWM à 16 MHz
uint16_t ct = 1600; // 10 kHz fréquence des bits
float T = 100; // période des bits en microsecondes (pour le décodage)
uint16_t refresh = 1; 
uint16_t boucle = 100; // nombre de répétitions périodiques de la séquence de bits

NRF52_PWM pwm;
mbed::InterruptIn inter(digitalPinToPinName(INPUT_PIN));
mbed::Timer temps;

// gestionnaire d'interruption pour les fronts montants
void f_rise() {
   t2 = temps.read_us();
   intervalles[indice_t] = t2-t1;
   t1 = t2;
   indice_t += 1;
   if (indice_t==NSAMPLES) indice_t = 0;  
}
// gestionnaire d'interruption pour les fronts descendants
void f_fall() {
   t2 = temps.read_us();
   intervalles[indice_t] = -(t2-t1);
   t1 = t2;
   indice_t += 1;
   if (indice_t==NSAMPLES) indice_t = 0;
}

// remplissage des deux tableaux pour le PWM
void remplissage_sequences() {
  for (int i=0; i<NBITS; i++) {
    if (bits[i]==1) {
      sequence_0[i] = 0;
      sequence_1[i] = 0;
    }
    else {
      sequence_0[i] = ct;
      sequence_1[i] = ct;
    }
  }
}

void setup() {
  Serial.begin(115200);
  pinMode(OUTPUT_PIN,OUTPUT);
  pinMode(INPUT_PIN,INPUT);
  remplissage_sequences();
  indice_t = 0;
  inter.rise(&f_rise);
  inter.fall(&f_fall);
  temps.start();
  t1 = temps.read_us();
}

void loop() {
   pwm.modulation_une_sortie(OUTPUT_PIN,prescaler,refresh,NBITS,boucle,ct,sequence_0,sequence_1);
   pwm.attendre();
  int j = 0;
  uint16_t nb;
  for (int i=0; i<NSAMPLES; i++) {
   nb = round(abs(intervalles[i])/T); // rapport duree de l'intervalle sur duree d'un bit
#ifdef DECODE_BITS
  if (intervalles[i]>0) for (int k=0; k<nb; k++) Serial.print("0");
  else for (int k=0; k<nb; k++) Serial.print("1");
#else
  Serial.println(intervalles[i]);
#endif 
  }
}
    	         
    	         